@@ -1,5 +1,5 @@
 /** BEGIN COPYRIGHT BLOCK
- * Copyright (C) 2020 Red Hat, Inc.
+ * Copyright (C) 2023 Red Hat, Inc.
  * All rights reserved.
  *
  * License: GPL (version 3 or any later version).
@@ -886,7 +886,7 @@ dbmdb_backup(struct ldbminfo *li, char *dest_dir, Slapi_Task *task)
 
     if ('\0' == *home) {
         slapi_log_err(SLAPI_LOG_ERR,
-                      "dblayer_backup", "Missing db home directory info\n");
+                      "dbmdb_backup", "Missing db home directory info\n");
         return return_value;
     }
 
@@ -899,7 +899,7 @@ dbmdb_backup(struct ldbminfo *li, char *dest_dir, Slapi_Task *task)
      */
 
     if (g_get_shutdown() || c_get_shutdown()) {
-        slapi_log_err(SLAPI_LOG_WARNING, "dblayer_backup", "Server shutting down, backup aborted\n");
+        slapi_log_err(SLAPI_LOG_WARNING, "dbmdb_backup", "Server shutting down, backup aborted\n");
         return_value = -1;
         goto bail;
     }
@@ -936,14 +936,14 @@ dbmdb_backup(struct ldbminfo *li, char *dest_dir, Slapi_Task *task)
     /* now copy the info file */
     pathname1 = slapi_ch_smprintf("%s/%s", home, INFOFILE);
     pathname2 = slapi_ch_smprintf("%s/%s", dest_dir, INFOFILE);
-    slapi_log_err(SLAPI_LOG_INFO, "dblayer_backup", "Backing up file d (%s)\n", pathname2);
+    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_backup", "Backing up file d (%s)\n", pathname2);
     if (task) {
         slapi_task_log_notice(task, "Backing up file (%s)", pathname2);
     }
     return_value = dbmdb_copyfile(pathname1, pathname2, 0, li->li_mode | 0400);
     if (0 > return_value) {
         slapi_log_err(SLAPI_LOG_ERR,
-                      "dblayer_backup", "Error in copying version file "
+                      "dbmdb_backup", "Error in copying version file "
                                         "(%s -> %s): err=%d\n",
                       pathname1, pathname2, return_value);
         if (task) {
@@ -958,7 +958,15 @@ dbmdb_backup(struct ldbminfo *li, char *dest_dir, Slapi_Task *task)
 
     if (0 == return_value) /* if everything went well, backup the index conf */
         return_value = dbmdb_dse_conf_backup(li, dest_dir);
+
+    /* Backup the config files */
+    if (ldbm_archive_config(dest_dir, task) != 0) {
+        slapi_log_err(SLAPI_LOG_ERR, "dbmdb_backup",
+                "Backup of config files failed or is incomplete\n");
+    }
+
     goto bail;
+
 error_out:
     slapi_log_err(SLAPI_LOG_ERR, "dbmdb_backup", "Backup to %s aborted.\n", dest_dir);
     if (task) {
@@ -1025,8 +1033,8 @@ dbmdb_restore(struct ldbminfo *li, char *src_dir, Slapi_Task *task)
     /* We check on the source staging area, no point in going further if it
      * isn't there */
     if (stat(src_dir, &sbuf) < 0) {
-        slapi_log_err(SLAPI_LOG_ERR, "dbmdb_restore", "Backup directory %s does not "
-                                                        "exist.\n",
+        slapi_log_err(SLAPI_LOG_ERR, "dbmdb_restore",
+                      "Backup directory %s does not exist.\n",
                       src_dir);
         if (task) {
             slapi_task_log_notice(task, "Restore: backup directory %s does not exist.",
@@ -1545,7 +1553,7 @@ dbi_error_t dbmdb_map_error(const char *funcname, int err)
             }
             slapi_log_err(SLAPI_LOG_ERR, "dbmdb_map_error",
                 "%s failed with db error %d : %s\n", funcname, err, msg);
-            log_stack(SLAPI_LOG_ERR);
+            slapi_log_backtrace(SLAPI_LOG_ERR);
             return DBI_RC_OTHER;
     }
 }
@@ -2550,11 +2558,11 @@ int dbmdb_public_new_cursor(dbi_db_t *db,  dbi_cursor_t *cursor)
            rc = MDB_NOTFOUND;
         } else if (rc==EINVAL) {
             slapi_log_err(SLAPI_LOG_ERR, "dbmdb_public_new_cursor", "Invalid dbi =%d (%s) while opening cursor in txn= %p\n", dbi->dbi, dbi->dbname, TXN(cursor->txn));
-            log_stack(SLAPI_LOG_ERR);
+            slapi_log_backtrace(SLAPI_LOG_ERR);
         } else {
             rc = EINVAL;
             slapi_log_err(SLAPI_LOG_ERR, "dbmdb_public_new_cursor", "Failed to open cursor dbi =%d (%s) in txn= %p\n", dbi->dbi, dbi->dbname, TXN(cursor->txn));
-            log_stack(SLAPI_LOG_ERR);
+            slapi_log_backtrace(SLAPI_LOG_ERR);
          }
     }
     if (rc && cursor->islocaltxn)
@@ -2933,3 +2941,48 @@ error:
     *flag_err = rc;
     return idl;
 }
+
+int
+dbmdb_dblayer_cursor_iterate(dbi_cursor_t *cursor, dbi_iterate_cb_t *action_cb,
+                             const dbi_val_t *startingkey, void *ctx)
+{
+    MDB_cursor *mdb_cur = (MDB_cursor*)cursor->cur;
+    MDB_val mdb_key = {0};
+    MDB_val mdb_data = {0};
+    dbi_val_t key = {0};
+    dbi_val_t data = {0};
+    int rc = 0;
+
+    if (mdb_cur == NULL) {
+        return  DBI_RC_INVALID;
+    }
+
+    if (startingkey && startingkey->data && startingkey->size) {
+        mdb_key.mv_data = startingkey->data;
+        mdb_key.mv_size = startingkey->size;
+        rc = MDB_CURSOR_GET(mdb_cur, &mdb_key, &mdb_data, MDB_SET_RANGE);
+    } else {
+        rc = MDB_CURSOR_GET(mdb_cur, &mdb_key, &mdb_data, MDB_FIRST);
+    }
+    while (rc == MDB_SUCCESS) {
+        key.data = mdb_key.mv_data;
+        key.size = mdb_key.mv_size;
+        data.data = mdb_data.mv_data;
+        data.size = mdb_data.mv_size;
+        rc = action_cb(&key, &data, ctx);
+        if (rc == DBI_RC_NOTFOUND) {
+            rc = DBI_RC_SUCCESS;
+            break;
+        }
+        rc = MDB_CURSOR_GET(mdb_cur, &mdb_key, &mdb_data, MDB_NEXT);
+    }
+    if (rc == MDB_NOTFOUND) {
+        rc = DBI_RC_NOTFOUND;
+    } else if (rc != DBI_RC_SUCCESS) {
+        slapi_log_err(SLAPI_LOG_ERR, "dbmdb_dblayer_cursor_iterate",
+                      "Database error while iterating a cursor ; db error - %d %s\n",
+                      rc, mdb_strerror(rc));
+        rc = dbmdb_map_error(__FUNCTION__, rc);
+    }
+    return rc;
+}
