@@ -348,12 +348,13 @@ int config_set_pw_is_legacy_policy(const char *attrname, char *value, char *erro
 int config_set_pw_track_last_update_time(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_pw_gracelimit(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_pw_admin_dn(const char *attrname, char *value, char *errorbuf, int apply);
+int32_t config_set_pw_admin_skip_info(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_pw_send_expiring(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_useroc(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_return_exact_case(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_result_tweak(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_referral_mode(const char *attrname, char *url, char *errorbuf, int apply);
-int config_set_conntablesize(const char *attrname, char *url, char *errorbuf, int apply);
+int config_set_num_listeners(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_maxbersize(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_maxsasliosize(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_versionstring(const char *attrname, char *versionstring, char *errorbuf, int apply);
@@ -375,6 +376,7 @@ int config_set_rewrite_rfc1274(const char *attrname, char *value, char *errorbuf
 int config_set_outbound_ldap_io_timeout(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_unauth_binds_switch(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_require_secure_binds(const char *attrname, char *value, char *errorbuf, int apply);
+int config_set_close_on_failed_bind(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_anon_access_switch(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_localssf(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_minssf(const char *attrname, char *value, char *errorbuf, int apply);
@@ -515,13 +517,13 @@ long long config_get_pw_minage(void);
 long long config_get_pw_warning(void);
 int config_get_errorlog_level(void);
 int config_get_accesslog_level(void);
-int config_get_statlog_level();
+int config_get_statlog_level(void);
 int config_get_securitylog_level(void);
 int config_get_auditlog_logging_enabled(void);
 int config_get_auditfaillog_logging_enabled(void);
 char *config_get_auditlog_display_attrs(void);
 char *config_get_referral_mode(void);
-int config_get_conntablesize(void);
+int config_get_num_listeners(void);
 int config_check_referral_mode(void);
 ber_len_t config_get_maxbersize(void);
 int32_t config_get_maxsasliosize(void);
@@ -549,6 +551,7 @@ int config_get_rewrite_rfc1274(void);
 int config_get_outbound_ldap_io_timeout(void);
 int config_get_unauth_binds_switch(void);
 int config_get_require_secure_binds(void);
+int config_get_close_on_failed_bind(void);
 int config_get_anon_access_switch(void);
 int config_get_localssf(void);
 int config_get_minssf(void);
@@ -629,6 +632,9 @@ int32_t config_set_ldapssotoken_secret(const char *attrname, char *value, char *
 int32_t config_set_ldapssotoken_ttl(const char *attrname, char *value, char *errorbuf, int apply);
 int32_t config_get_ldapssotoken_ttl(void);
 
+int32_t config_get_referral_check_period(void);
+int32_t config_set_referral_check_period(const char *attrname, char *value, char *errorbuf, int apply);
+
 int32_t config_get_return_orig_dn(void);
 int32_t config_set_return_orig_dn(const char *attrname, char *value, char *errorbuf, int apply);
 
