@@ -1405,6 +1405,12 @@ void modify_update_last_modified_attr(Slapi_PBlock *pb, Slapi_Mods *smods);
 /* add.c */
 void add_internal_modifiersname(Slapi_PBlock *pb, Slapi_Entry *e);
 
+/* dse.c */
+void dse_init_backup_lock(void);
+void dse_destroy_backup_lock(void);
+void dse_backup_lock(void);
+void dse_backup_unlock(void);
+
 /* ldaputil.c */
 char *ldaputil_get_saslpath(void);
 int slapi_client_uses_non_nss(LDAP *ld);
@@ -1508,6 +1514,8 @@ void slapi_pblock_set_task_warning(Slapi_PBlock *pb, task_warning warning);
 
 int slapi_exists_or_add_internal(Slapi_DN *dn, const char *filter, const char *entry, const char *modifier_name);
 
+void slapi_log_backtrace(int loglevel);
+
 #ifdef __cplusplus
 }
 #endif
