@@ -1,4 +1,6 @@
+
 import cockpit from 'cockpit';
+import React from 'react';
 import {
     Label,
 } from '@patternfly/react-core';
@@ -71,7 +73,7 @@ export function getUserSuffixes (serverId, suffixCallback) {
   ];
   log_cmd("getUserSuffixes", "list suffixes", suffixCmd);
   cockpit
-    .spawn(suffixCmd, { superuser: true })
+    .spawn(suffixCmd, { superuser: true, err: "message" })
     .done(content => {
       const suffList = JSON.parse(content);
       suffixCallback(suffList.items)
@@ -148,7 +150,7 @@ export function getRootSuffixEntryDetails (params, entryDetailsCallback) {
   let modifyTimestamp = '';
   // let entryId = '';
   let result = {};
-  const entryArray = []; // Will contain the entry but the numSubordinates and modifyTimestamp.
+  let entryArray = []; // Will contain the entry but the numSubordinates and modifyTimestamp.
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' })
     .done(data => {
@@ -209,7 +211,7 @@ export function getRootSuffixEntryDetails (params, entryDetailsCallback) {
 function getResourceLimits () {
   const timeLimit = getTimeLimit();
   const sizeLimit = getSizeLimit();
-  const limits = [];
+  let limits = [];
   if (timeLimit > 0) {
     limits.push('-l', timeLimit);
   }
@@ -256,7 +258,7 @@ export function getSearchEntries (params, resultCallback) {
   let numSubordinates = '0';
   let modifyTimestamp = '';
   let searchResult = null;
-  const allEntries = [];
+  let allEntries = [];
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' }) // string.split("\n\r")
     .done(data => {
@@ -384,7 +386,7 @@ export function getBaseLevelEntryAttributes (serverId, baseDn, entryAttributesCa
   */
 
   log_cmd("getBaseLevelEntryAttributes", "", cmd);
-  const entryArray = [];
+  let entryArray = [];
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' })
     .done(data => {
@@ -395,8 +397,7 @@ export function getBaseLevelEntryAttributes (serverId, baseDn, entryAttributesCa
           if (currentLine.length < 1000 || currentLine.substring(0, 9).toLowerCase().startsWith("jpegphoto")) {
             entryArray.push(splitAttributeValue(currentLine));
           } else {
-            const myTruncatedValue = (<div>
-                                          currentLine.substring(0, 1000)
+            const myTruncatedValue = (<div name="truncated" attr={splitAttributeValue(currentLine).attribute}>
                                           <Label icon={<InfoCircleIcon />} color="blue" >
                                               Value is too large to display
                                           </Label>
@@ -452,7 +453,7 @@ export function getBaseLevelEntryFullAttributes (serverId, baseDn, entryAttribut
   ];
 
   log_cmd("getBaseLevelEntryFullAttributes", "", cmd);
-  const entryArray = [];
+  let entryArray = [];
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' })
     .done(data => {
@@ -527,7 +528,7 @@ export function getOneLevelEntries (params, oneLevelCallback) {
   let numSubordinates = '';
   let modifyTimestamp = '';
   let searchResult = null;
-  const allEntries = [];
+  let allEntries = [];
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' }) // string.split("\n\r")
     .done(data => {
@@ -764,7 +765,7 @@ export function listAccessLogs (logDirectory, logListCallback) {
 
   log_cmd("listAccessLogs", "", cmd);
 
-  const logDataArray = [];
+  let logDataArray = [];
 
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' })
@@ -870,7 +871,7 @@ export function getAllObjectClasses (serverId, allOcCallback) {
     'objectclasses',
     'list'
   ];
-  const result = [];
+  let result = [];
   log_cmd("getAllObjectClasses", "", cmd);
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' })
@@ -902,7 +903,7 @@ export function getSingleValuedAttributes (serverId, svCallback) {
     'attributetypes',
     'list'
   ];
-  const result = [];
+  let result = [];
   log_cmd("getSingleValuedAttributes", "", cmd);
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' })
@@ -932,7 +933,7 @@ export function getAttributesNameAndOid (serverId, attrCallback) {
     'attributetypes',
     'list'
   ];
-  const result = [];
+  let result = [];
   log_cmd("getAttributesNameAndOid", "", cmd);
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' })
@@ -951,7 +952,7 @@ export function getAttributesNameAndOid (serverId, attrCallback) {
 }
 
 export function deleteLdapData (serverId, entryDN, numSubordinates, deleteCallback) {
-  const cmd = [
+  let cmd = [
     'ldapdelete',
     '-Y',
     'EXTERNAL',
@@ -997,7 +998,7 @@ export function showCertificate (certificate, showCertCallback) {
   ];
 
   let result = {};
-  const certDataArray = [];
+  let certDataArray = [];
 
   cockpit
     .spawn(cmd, { superuser: true, err: 'message' })
@@ -1043,7 +1044,7 @@ export function showCertificate (certificate, showCertCallback) {
 export function b64DecodeUnicode (str) {
   // Going backwards: from bytestream, to percent-encoding, to original string.
   try {
-      result = decodeURIComponent(atob(str).split('').map(c => {
+      let result = decodeURIComponent(atob(str).split('').map(c => {
           return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
       }).join(''));
       return result;
@@ -1130,7 +1131,7 @@ export function generateRootEntryLdif (suffixDn, ldifCallback) {
       console.log(`The RDN attribute ${myRdnAttr} is not supported for root entry creation!`);
   }
 
-  const ldifArray = [];
+  let ldifArray = [];
   if (isValid) {
     ldifArray.push(`dn: ${suffixDn}`);
     ldifArray.push('objectClass: top');
@@ -1146,7 +1147,7 @@ export function generateRootEntryLdif (suffixDn, ldifCallback) {
 export function retrieveAllAcis (params, aciCallback) {
     let entryDn = params.baseDn;
     const potentialEntries = [params.baseDn];
-    const resultArray = []; // An item will be like {entryDn:<DN>, aciArray:[aci:<ACI_1>,aci:<ACI_2>]}
+    let resultArray = []; // An item will be like {entryDn:<DN>, aciArray:[aci:<ACI_1>,aci:<ACI_2>]}
     const myParams = {
         serverId: params.serverId,
         baseDn: entryDn,
@@ -1232,7 +1233,7 @@ export function foldLine (line) {
   }
 
   const size = Math.ceil(line.length / LDIF_MAX_CHAR_PER_LINE)
-  const lineArray = new Array(size);
+  let lineArray = new Array(size);
   // Handle the first line separately since it doesn't start with a space.
   lineArray[0] = line.substring(0, LDIF_MAX_CHAR_PER_LINE);
 
@@ -1250,3 +1251,12 @@ export function isValidLDAPUrl (url) {
     }
     return false;
 }
+
+export function getBaseDNFromTree (entrydn, treeViewRootSuffixes) {
+    for (const suffixObj of treeViewRootSuffixes) {
+        if (entrydn.toLowerCase().indexOf(suffixObj.name.toLowerCase()) !== -1) {
+            return suffixObj.name;
+        }
+    }
+    return "";
+}
