@@ -5,11 +5,8 @@ import {
     Bullseye,
     Card, CardHeader, CardBody, CardTitle,
     Dropdown, DropdownItem, DropdownPosition,
-    FormSelect, FormSelectOption,
     Grid, GridItem,
     Label, LabelGroup,
-    Modal,
-    ModalVariant,
     Pagination,
     SearchInput,
     SimpleList, SimpleListItem,
@@ -36,6 +33,7 @@ import {
     modifyLdapEntry,
 } from '../../lib/utils.jsx';
 import EditableTable from '../../lib/editableTable.jsx';
+import EditGroup from './editGroup.jsx';
 import {
     LDAP_OPERATIONS,
     BINARY_ATTRIBUTES,
@@ -95,6 +93,13 @@ class EditLdapEntry extends React.Component {
             selectedAttributes: [],
             attrsToRemove: [],
             modifying: true,
+            isGroupOfNames: false,
+            isGroupOfUniqueNames: false,
+            groupMembers: [],
+            groupGenericEditor: false,
+            localProps: {...this.props},
+            searchOCValue: "",
+            searchValue: "",
         };
 
         this.onNext = ({ id }) => {
@@ -114,7 +119,7 @@ class EditLdapEntry extends React.Component {
                 // Generate the LDIF data at step 4.
                 this.generateLdifData();
             } else if (id === 6) {
-                const params = { serverId: this.props.editorLdapServer };
+                const params = { serverId: this.state.localProps.editorLdapServer };
                 modifyLdapEntry(params, this.state.ldifArray, (result) => {
                     if (result.errorCode === 0) {
                         result.output = "Successfully modified entry"
@@ -124,13 +129,13 @@ class EditLdapEntry extends React.Component {
                         commandOutput: result.errorCode === 0 ? 'Successfully modified entry!' : 'Failed to modify entry, error: ' + result.errorCode ,
                         resultVariant: result.errorCode === 0 ? 'success' : 'danger',
                         modifying: false,
-                    }, () => { this.props.onReload() }); // refreshes tableView
+                    }, () => { this.state.localProps.onReload() }); // refreshes tableView
                     const opInfo = { // This is what refreshes treeView
                         operationType: 'MODIFY',
                         resultCode: result.errorCode,
                         time: Date.now()
                     }
-                    this.props.setWizardOperationInfo(opInfo);
+                    this.state.localProps.setWizardOperationInfo(opInfo);
                 });
             }
         };
@@ -160,7 +165,7 @@ class EditLdapEntry extends React.Component {
             const val = value.toLowerCase();
 
             // Get fresh list of Objectclasses and what is selected
-            this.props.allObjectclasses.map(oc => {
+            this.state.localProps.allObjectclasses.map(oc => {
                 let selected = false;
                 let selectionDisabled = false;
                 for (const selectedOC of this.state.selectedObjectClasses) {
@@ -203,6 +208,8 @@ class EditLdapEntry extends React.Component {
             this.setState({
                 rowsOc: ocRows,
                 pagedRowsOc: ocRows.slice(0, this.state.perPageOc),
+                itemCountOc: ocRows.length,
+                searchOCValue: value
             })
         }
 
@@ -230,12 +237,34 @@ class EditLdapEntry extends React.Component {
             this.setState({
                 rowsAttr: attrRows,
                 pagedRowsAttr: attrRows.slice(0, this.state.perPageAttr),
+                itemCountAttr: attrRows.length,
+                searchValue: value,
             })
         }
 
+        this.useGroupGenericEditor = this.useGroupGenericEditor.bind(this);
+        this.loadEntry = this.loadEntry.bind(this);
         // End constructor().
     }
 
+    useGroupGenericEditor = () => {
+        this.originalEntryRows = [];
+        this.setState({
+            groupGenericEditor: true
+        });
+    }
+
+    openEditEntry = (dn) => {
+        // used by group modal
+        let editProps = { ...this.state.localProps};
+        editProps.wizardEntryDn = dn;
+        this.setState({
+            localProps: editProps,
+            isGroupOfNames: false,
+            isGroupOfUniqueNames: false,
+        })
+    }
+
     isAttributeSingleValued = (attr) => {
         return this.singleValuedAttributes.includes(attr.toLowerCase());
     };
@@ -265,86 +294,111 @@ class EditLdapEntry extends React.Component {
         });
     }
 
-    componentDidMount () {
+    loadEntry(reload) {
         const ocArray = [];
-        getSingleValuedAttributes(this.props.editorLdapServer,
-            (myAttrs) => {
-                this.singleValuedAttributes = [...myAttrs];
-        });
+        if (reload) {
+            this.originalEntryRows = [];
+        }
 
-        getBaseLevelEntryAttributes(this.props.editorLdapServer,
-            this.props.wizardEntryDn,
+        getBaseLevelEntryAttributes(this.state.localProps.editorLdapServer,
+            this.state.localProps.wizardEntryDn,
             (entryDetails) => {
                 let objectclasses = [];
-                const rdnInfo = getRdnInfo(this.props.wizardEntryDn);
+                const rdnInfo = getRdnInfo(this.state.localProps.wizardEntryDn);
                 let namingAttr = "";
                 let namingValue = "";
                 let namingIndex = -1;
                 let attrPropsName = "";
+                let isGroupOfUniqueNames = false;
+                let isGroupOfNames = false;
+                let members = [];
 
                 entryDetails
                 .filter(data => (data.attribute + data.value !== '' && // Filter out empty lines
                 data.attribute !== '???: ')) // and data for empty suffix(es) and in case of failure.
                 .map((line, index) => {
-                    const obj = {};
-                    const attr = line.attribute;
-                    const attrLowerCase = attr.trim().toLowerCase();
+                    let attrLowerCase;
                     let namingAttribute = false;
-                    let val = line.value.substring(1).trim();
-                    let encodedvalue = "";
+                    if (line.attribute !== undefined) {
+                        const obj = {};
+                        const attr = line.attribute;
+                        attrLowerCase = attr.trim().toLowerCase();
+                        let val = line.value.substring(1).trim();
+                        let encodedvalue = "";
 
-                    if (attrLowerCase === "objectclass") {
-                        objectclasses.push(val);
-                    } else {
-                        // Base64 encoded values
-                        if (line.attribute === "dn") {
-                            //return;
-                        }
-                        if (line.value.substring(0, 2) === '::') {
-                            val = line.value.substring(3);
-                            if (BINARY_ATTRIBUTES.includes(attrLowerCase)) {
-                                // obj.fileUpload = true;
-                                // obj.isDisabled = true;
-                                if (attrLowerCase === 'jpegphoto') {
-                                    const myPhoto = (<img
-                                        src={`data:image/png;base64,${val}`}
-                                        alt=""
-                                        style={{ width: '48px' }} // height will adjust automatically.
-                                        />);
-                                    encodedvalue = val;
-                                    val = myPhoto;
-                                } else if (attrLowerCase === 'nssymmetrickey') {
-                                    // TODO: Check why the decoding of 'nssymmetrickey is failing...
-                                    //   https://access.redhat.com/documentation/en-us/red_hat_directory_server/10
-                                    //   /html/configuration_command_and_file_reference/core_server_configuration_reference#cnchangelog5-nsSymmetricKey
-                                    //
-                                    // Just show the encoded value at the moment.
-                                    val = line.value.substring(3);
-                                }
-                            } else { // The value likely contains accented characters or has a trailing space.
-                                val = b64DecodeUnicode(line.value.substring(3));
+                        if (attrLowerCase === "objectclass") {
+                            objectclasses.push(val);
+                            if (val.toLowerCase() === "groupofnames") {
+                                isGroupOfNames = true;
+                            } else if (val.toLowerCase() === "groupofuniquenames") {
+                                isGroupOfUniqueNames = true;
                             }
                         } else {
-                            // Check for naming attribute
-                            if (attr === rdnInfo.rdnAttr && val === rdnInfo.rdnVal) {
-                                namingAttribute = true;
-                                namingAttr = attr;
-                                namingValue = val;
+                            // Base64 encoded values
+                            if (line.attribute === "dn") {
+                                //return;
+                            }
+                            if (line.value.substring(0, 2) === '::') {
+                                val = line.value.substring(3);
+                                if (BINARY_ATTRIBUTES.includes(attrLowerCase)) {
+                                    // obj.fileUpload = true;
+                                    // obj.isDisabled = true;
+                                    if (attrLowerCase === 'jpegphoto') {
+                                        const myPhoto = (<img
+                                            src={`data:image/png;base64,${val}`}
+                                            alt=""
+                                            style={{ width: '48px' }} // height will adjust automatically.
+                                            />);
+                                        encodedvalue = val;
+                                        val = myPhoto;
+                                    } else if (attrLowerCase === 'nssymmetrickey') {
+                                        // TODO: Check why the decoding of 'nssymmetrickey is failing...
+                                        //   https://access.redhat.com/documentation/en-us/red_hat_directory_server/10
+                                        //   /html/configuration_command_and_file_reference/core_server_configuration_reference#cnchangelog5-nsSymmetricKey
+                                        //
+                                        // Just show the encoded value at the moment.
+                                        val = line.value.substring(3);
+                                    }
+                                } else { // The value likely contains accented characters or has a trailing space.
+                                    val = b64DecodeUnicode(line.value.substring(3));
+                                }
+                            } else {
+                                // Check for naming attribute
+                                if (attr === rdnInfo.rdnAttr && val === rdnInfo.rdnVal) {
+                                    namingAttribute = true;
+                                    namingAttr = attr;
+                                    namingValue = val;
+                                }
                             }
-                        }
 
+                            obj.id = generateUniqueId();
+                            obj.attr = attr;
+                            obj.val = val;
+                            obj.encodedvalue = encodedvalue;
+                            obj.namingAttr = namingAttribute;
+                            obj.required = namingAttribute;
+                            this.originalEntryRows.push(obj);
+                        }
+                        // Handle group members separately
+                        if (attrLowerCase === "member" || attrLowerCase === "uniquemember") {
+                            members.push(val);
+                        }
+                    } else {
+                        // Value too large Label
+                        const obj = {};
                         obj.id = generateUniqueId();
-                        obj.attr = attr;
-                        obj.val = val;
-                        obj.encodedvalue = encodedvalue;
-                        obj.namingAttr = namingAttribute;
-                        obj.required = namingAttribute;
+                        obj.attr = line.props.attr;
+                        obj.val = line;
+                        obj.encodedvalue = "";
+                        obj.namingAttr = false;
+                        obj.required = false;
+                        attrLowerCase = line.props.attr.trim().toLowerCase();
                         this.originalEntryRows.push(obj);
                     }
                 });
 
                 // Mark the existing objectclass classes as selected
-                this.props.allObjectclasses.map(oc => {
+                this.state.localProps.allObjectclasses.map(oc => {
                     let selected = false;
                     let selectionDisabled = false;
                     for (const entryOC of objectclasses) {
@@ -388,6 +442,9 @@ class EditLdapEntry extends React.Component {
                     namingValue: namingValue,
                     origAttrs: JSON.parse(JSON.stringify(this.originalEntryRows)),
                     origOC: JSON.parse(JSON.stringify(selectedObjectClasses)),
+                    isGroupOfNames: isGroupOfNames,
+                    isGroupOfUniqueNames: isGroupOfUniqueNames,
+                    groupMembers: members.sort(),
                     loading: false,
                 }, () => {
                     this.updateAttributeTableRows();
@@ -395,6 +452,16 @@ class EditLdapEntry extends React.Component {
         });
     }
 
+    componentDidMount () {
+        getSingleValuedAttributes(this.props.editorLdapServer,
+            (myAttrs) => {
+                this.singleValuedAttributes = [...myAttrs];
+        });
+        this.setState({
+            localProps:  {...this.props}
+        }, () => { this.loadEntry() });
+    }
+
     onSetPageOc = (_event, pageNumber) => {
         this.setState({
             pageOc: pageNumber,
@@ -557,15 +624,15 @@ class EditLdapEntry extends React.Component {
 
     getAllAttrs = () => {
         const ocToProcess = [...this.state.selectedObjectClasses];
-        const rowsAttr = [];
-        const attrList = [];
+        let rowsAttr = [];
+        let attrList = [];
 
         ocToProcess.map(oc => {
             // Rebuild the attribute arrays.
             const required = oc.cells[1].split(',');
             const optional = oc.cells[2].split(',');
 
-            for (const attr of required) {
+            for (let attr of required) {
                 attr = attr.trim().toLowerCase();
                 if (attr === '') {
                     continue;
@@ -607,7 +674,7 @@ class EditLdapEntry extends React.Component {
                 }
             }
 
-            for (const attr of optional) {
+            for (let attr of optional) {
                 attr = attr.trim();
                 if (attr === '') {
                     continue;
@@ -635,7 +702,7 @@ class EditLdapEntry extends React.Component {
             let personOC = false;
             for (const existingOC of this.state.selectedObjectClasses) {
                 if (existingOC.cells[0].toLowerCase() === 'person' ||
-                   existingOC.cells[0].toLowerCase() === 'nsperson') {
+                    existingOC.cells[0].toLowerCase() === 'nsperson') {
                     personOC = true;
                     break;
                 }
@@ -688,11 +755,11 @@ class EditLdapEntry extends React.Component {
     };
 
     generateLdifData = () => {
-        const statementRows = [];
-        const ldifArray = [];
-        const updateArray = [];
-        const addArray = [];
-        const removeArray = [];
+        let statementRows = [];
+        let updateArray = [];
+        let addArray = [];
+        let removeArray = [];
+        let ldifArray = [];
         let cleanLdifArray = [];
         let numOfChanges = 0;
         let isFilePath = false;
@@ -745,9 +812,9 @@ class EditLdapEntry extends React.Component {
             found = false;
         }
 
-        for (const datum of updateArray) {
+        for (let datum of updateArray) {
             const myAttr = datum.attr;
-            const myVal = datum.val;
+            let myVal = datum.val;
             const isUserPwd = myAttr.toLowerCase() === "userpassword";
             isFilePath = false;
 
@@ -913,7 +980,7 @@ class EditLdapEntry extends React.Component {
         // Loop delete rows
         for (const datum of removeArray) {
             const myAttr = datum.attr;
-            const myVal = datum.val;
+            let myVal = datum.val;
             const isUserPwd = myAttr.toLowerCase() === "userpassword";
             isFilePath = false;
             // Update LDIF array
@@ -1152,7 +1219,7 @@ class EditLdapEntry extends React.Component {
                     <div>
                         <Bullseye className="ds-margin-top-xlg" key="add-entry-bulleye" >
                             <Title headingLevel="h3" size="lg" key="loading-title">
-                                Loading ObjectClasses ...
+                                Loading ...
                             </Title>
                         </Bullseye>
                         <Spinner className="ds-center" size="lg" key="loading-spinner" />
@@ -1164,7 +1231,7 @@ class EditLdapEntry extends React.Component {
                             <SearchInput
                                 className="ds-font-size-md"
                                 placeholder='Search Objectclasses'
-                                value={this.state.searchValue}
+                                value={this.state.searchOCValue}
                                 onChange={this.onOCSearchChange}
                                 onClear={(evt) => this.onOCSearchChange('', evt)}
                             />
@@ -1257,14 +1324,14 @@ class EditLdapEntry extends React.Component {
                 </TextContent>
                 <EditableTable
                     key={editableTableData}
-                    wizardEntryDn={this.props.wizardEntryDn}
+                    wizardEntryDn={this.state.localProps.wizardEntryDn}
                     editableTableData={editableTableData}
                     quickUpdate
                     isAttributeSingleValued={this.isAttributeSingleValued}
                     isAttributeRequired={this.isAttributeRequired}
                     enableNextStep={this.enableNextStep}
                     saveCurrentRows={this.saveCurrentRows}
-                    allObjectclasses={this.props.allObjectclasses}
+                    allObjectclasses={this.state.localProps.allObjectclasses}
                     disableNamingChange
                 />
             </>
@@ -1404,18 +1471,40 @@ class EditLdapEntry extends React.Component {
         ];
 
         const title = <>
-            Entry DN: &nbsp;&nbsp;<strong>{this.props.wizardEntryDn}</strong>
+            Entry DN: &nbsp;&nbsp;<strong>{this.state.localProps.wizardEntryDn}</strong>
         </>;
 
-        return (
+        let editPage =
             <Wizard
-                isOpen={this.props.isWizardOpen}
-                onClose={this.props.toggleOpenWizard}
+                isOpen={this.state.localProps.isWizardOpen}
+                onClose={this.state.localProps.toggleOpenWizard}
                 steps={editEntrySteps}
                 title="Edit An LDAP Entry"
                 description={title}
                 onNext={this.onNext}
-            />
+            />;
+
+        if (!this.state.groupGenericEditor && (this.state.isGroupOfNames || this.state.isGroupOfUniqueNames)) {
+            editPage =
+                <EditGroup
+                    key={this.state.groupMembers}
+                    groupdn={this.state.localProps.wizardEntryDn}
+                    members={this.state.groupMembers}
+                    useGenericEditor={this.useGroupGenericEditor}
+                    isGroupOfNames={this.state.isGroupOfNames}
+                    isGroupOfUniqueNames={this.state.isGroupOfUniqueNames}
+                    treeViewRootSuffixes={this.state.localProps.treeViewRootSuffixes}
+                    editorLdapServer={this.state.localProps.editorLdapServer}
+                    addNotification={this.state.localProps.addNotification}
+                    openEditEntry={this.openEditEntry}
+                    onReload={this.loadEntry}
+                />;
+        }
+
+        return (
+            <>
+                {editPage}
+            </>
         );
     }
 }
