@@ -179,7 +179,7 @@ export class ServerAccessLog extends React.Component {
 
         // Check if a setting was changed, if so enable the save button
         for (const config_attr of config_attrs) {
-            if (attr === config_attr && this.state['_' + config_attr] !== value) {
+            if (attr === config_attr && this.state['_' + config_attr].toString() !== value.toString()) {
                 disableSaveBtn = false;
                 break;
             }
@@ -187,7 +187,7 @@ export class ServerAccessLog extends React.Component {
 
         // Now check for differences in values that we did not touch
         for (const config_attr of config_attrs) {
-            if (attr !== config_attr && this.state['_' + config_attr] !== this.state[config_attr]) {
+            if (attr !== config_attr && this.state['_' + config_attr].toString() !== this.state[config_attr].toString()) {
                 disableSaveBtn = false;
                 break;
             }
@@ -262,7 +262,7 @@ export class ServerAccessLog extends React.Component {
             config_attrs = exp_attrs;
         }
 
-        const cmd = [
+        let cmd = [
             'dsconf', '-j', "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
             'config', 'replace'
         ];
