@@ -74,6 +74,10 @@ export class Security extends React.Component {
             // Ciphers
             supportedCiphers: [],
             enabledCiphers: [],
+            // Certificate Signing Requests
+            serverCSRs: [],
+            // Orphan keys
+            serverOrphanKeys: [],
             // Config settings
             securityEnabled: false,
             requireSecureBinds: false,
@@ -105,9 +109,14 @@ export class Security extends React.Component {
 
         // Server Cert
         this.handleServerCertSelect = (event, selection) => {
+            let disableSaveBtn = !this.configChanged();
+            if (this.state._nssslpersonalityssl !== selection) {
+                disableSaveBtn = false;
+            }
             this.setState({
                 nssslpersonalityssl: selection,
-                isServerCertOpen: false
+                isServerCertOpen: false,
+                disableSaveBtn,
             });
         };
         this.handleServerCertToggle = isServerCertOpen => {
@@ -226,6 +235,7 @@ export class Security extends React.Component {
         this.loadSupportedCiphers = this.loadSupportedCiphers.bind(this);
         this.loadCerts = this.loadCerts.bind(this);
         this.loadCACerts = this.loadCACerts.bind(this);
+        this.loadCSRs = this.loadCSRs.bind(this);
         this.closeConfirmDisable = this.closeConfirmDisable.bind(this);
         this.enableSecurity = this.enableSecurity.bind(this);
         this.disableSecurity = this.disableSecurity.bind(this);
@@ -235,6 +245,7 @@ export class Security extends React.Component {
         this.onSelectToggle = this.onSelectToggle.bind(this);
         this.onSelectClear = this.onSelectClear.bind(this);
         this.handleTypeaheadChange = this.handleTypeaheadChange.bind(this);
+        this.onSecEnableChange = this.onSecEnableChange.bind(this);
     }
 
     componentDidMount () {
@@ -318,7 +329,7 @@ export class Security extends React.Component {
             "dsconf", "-j", "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
             "security", "ca-certificate", "list",
         ];
-        log_cmd("loadCACerts", "Load certificates", cmd);
+        log_cmd("loadCACerts", "Load CA certificates", cmd);
         cockpit
                 .spawn(cmd, { superuser: true, err: "message" })
                 .done(content => {
@@ -326,8 +337,7 @@ export class Security extends React.Component {
                     this.setState(() => (
                         {
                             CACerts: certs,
-                            loaded: true
-                        }), this.props.enableTree()
+                        }), this.loadCSRs
                     );
                 })
                 .fail(err => {
@@ -378,6 +388,67 @@ export class Security extends React.Component {
                 });
     }
 
+    loadCSRs () {
+        const cmd = [
+            "dsconf", "-j", "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
+            "security", "csr", "list",
+        ];
+        log_cmd("loadCSRs", "Load CSRs", cmd);
+        cockpit
+                .spawn(cmd, { superuser: true, err: "message" })
+                .done(content => {
+                    const csrs = JSON.parse(content);
+                    this.setState(() => (
+                        {
+                            serverCSRs: csrs,
+                        }), this.loadOrphanKeys
+                    );
+                })
+                .fail(err => {
+                    const errMsg = JSON.parse(err);
+                    let msg = errMsg.desc;
+                    if ('info' in errMsg) {
+                        msg = errMsg.desc + " - " + errMsg.info;
+                    }
+                    this.props.addNotification(
+                        "error",
+                        `Error loading CSRs - ${msg}`
+                    );
+                });
+    }
+
+    loadOrphanKeys () {
+        const cmd = [
+            "dsconf", "-j", "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
+            "security", "key", "list", "--orphan"
+        ];
+        log_cmd("loadOrphanKeys", "Load Orphan Keys", cmd);
+        cockpit
+                .spawn(cmd, { superuser: true, err: "message" })
+                .done(content => {
+                    const keys = JSON.parse(content);
+                    this.setState(() => (
+                        {
+                            serverOrphanKeys: keys,
+                            loaded: true
+                        }), this.props.enableTree()
+                    );
+                })
+                .fail(err => {
+                    const errMsg = JSON.parse(err);
+                    if (!errMsg.desc.includes('certutil: no keys found')) {
+                        this.props.addNotification(
+                            "error",
+                            `Error loading Orphan Keys - ${errMsg.desc}`
+                        );
+                    }
+                    this.setState({
+                        loaded: true,
+                        serverOrphanKeys: []
+                    }, this.props.enableTree());
+                });
+    }
+
     loadRSAConfig() {
         const cmd = [
             "dsconf", "-j", "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
@@ -493,6 +564,7 @@ export class Security extends React.Component {
                             _sslVersionMin: attrs.sslversionmin,
                             _sslVersionMax: attrs.sslversionmax,
                             _allowWeakCipher: allowWeak,
+                            disableSaveBtn: true,
                         }
                     ), function() {
                         if (!saving) {
@@ -563,23 +635,51 @@ export class Security extends React.Component {
             "dsconf", "-j", "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
             "security", "enable",
         ];
-        log_cmd("enableSecurity", "Enable security", cmd);
-        cockpit
-                .spawn(cmd, { superuser: true, err: "message" })
+
+        if (this.state._nssslpersonalityssl !== this.state.primaryCertName) {
+            const rsa_cmd = [
+                'dsconf', '-j', 'ldapi://%2fvar%2frun%2fslapd-' + this.props.serverId + '.socket',
+                'security', 'rsa', 'set', '--nss-cert-name=' + this.state.primaryCertName
+            ];
+            log_cmd("enableSecurity", "Update RSA", rsa_cmd);
+            cockpit
+                .spawn(rsa_cmd, { superuser: true, err: "message" })
                 .done(() => {
-                    this.props.addNotification(
-                        "success",
-                        `Successfully enabled security.`
-                    );
-                    this.props.addNotification(
-                        "warning",
-                        `You must restart the Directory Server for these changes to take effect.`
-                    );
-                    this.setState({
-                        securityEnabled: true,
-                        secEnableSpinner: false,
-                        showSecurityEnableModal: false,
-                    });
+                    this.loadSecurityConfig();
+                    log_cmd("enableSecurity", "Enable security", cmd);
+                    cockpit
+                            .spawn(cmd, { superuser: true, err: "message" })
+                            .done(() => {
+                                this.loadSecurityConfig();
+                                this.props.addNotification(
+                                    "success",
+                                    `Successfully enabled security.`
+                                );
+                                this.props.addNotification(
+                                    "warning",
+                                    `You must restart the Directory Server for these changes to take effect.`
+                                );
+                                this.setState({
+                                    securityEnabled: true,
+                                    secEnableSpinner: false,
+                                    showSecurityEnableModal: false,
+                                });
+                            })
+                            .fail(err => {
+                                const errMsg = JSON.parse(err);
+                                let msg = errMsg.desc;
+                                if ('info' in errMsg) {
+                                    msg = errMsg.desc + " - " + errMsg.info;
+                                }
+                                this.props.addNotification(
+                                    "error",
+                                    `Error enabling security - ${msg}`
+                                );
+                                this.setState({
+                                    secEnableSpinner: false,
+                                    showSecurityEnableModal: false,
+                                });
+                            });
                 })
                 .fail(err => {
                     const errMsg = JSON.parse(err);
@@ -589,13 +689,48 @@ export class Security extends React.Component {
                     }
                     this.props.addNotification(
                         "error",
-                        `Error enabling security - ${msg}`
+                        `Error enabling security (RSA cert name)- ${msg}`
                     );
                     this.setState({
                         secEnableSpinner: false,
                         showSecurityEnableModal: false,
                     });
                 });
+        } else {
+            log_cmd("enableSecurity", "Enable security", cmd);
+            cockpit
+                    .spawn(cmd, { superuser: true, err: "message" })
+                    .done(() => {
+                        this.props.addNotification(
+                            "success",
+                            `Successfully enabled security.`
+                        );
+                        this.props.addNotification(
+                            "warning",
+                            `You must restart the Directory Server for these changes to take effect.`
+                        );
+                        this.setState({
+                            securityEnabled: true,
+                            secEnableSpinner: false,
+                            showSecurityEnableModal: false,
+                        });
+                    })
+                    .fail(err => {
+                        const errMsg = JSON.parse(err);
+                        let msg = errMsg.desc;
+                        if ('info' in errMsg) {
+                            msg = errMsg.desc + " - " + errMsg.info;
+                        }
+                        this.props.addNotification(
+                            "error",
+                            `Error enabling security - ${msg}`
+                        );
+                        this.setState({
+                            secEnableSpinner: false,
+                            showSecurityEnableModal: false,
+                        });
+                    });
+        }
     }
 
     disableSecurity () {
@@ -666,6 +801,10 @@ export class Security extends React.Component {
             'dsconf', '-j', 'ldapi://%2fvar%2frun%2fslapd-' + this.props.serverId + '.socket',
             'security', 'set'
         ];
+        const rsa_cmd = [
+            'dsconf', '-j', 'ldapi://%2fvar%2frun%2fslapd-' + this.props.serverId + '.socket',
+            'security', 'rsa', 'set'
+        ];
 
         if (this.state._validateCert !== this.state.validateCert) {
             cmd.push("--verify-cert-chain-on-startup=" + this.state.validateCert);
@@ -712,6 +851,53 @@ export class Security extends React.Component {
             cmd.push("--tls-client-renegotiation=" + val);
         }
 
+        if (this.state._nssslpersonalityssl !== this.state.nssslpersonalityssl) {
+            rsa_cmd.push("--nss-cert-name=" + this.state.nssslpersonalityssl);
+        }
+        if (rsa_cmd.length > 6) {
+            log_cmd("saveSecurityConfig", "Applying security RSA config change", rsa_cmd);
+            const msg = "Successfully updated security RSA configuration.";
+
+            this.setState({
+                // Start the spinner
+                saving: true
+            });
+
+            cockpit
+                    .spawn(rsa_cmd, { superuser: true, err: "message" })
+                    .done(content => {
+                        this.loadSecurityConfig();
+                        if (cmd.length < 6) {
+                            this.props.addNotification(
+                                "success",
+                                msg
+                            );
+                            this.props.addNotification(
+                                "warning",
+                                `You must restart the Directory Server for these changes to take effect.`
+                            );
+                            this.setState({
+                                saving: false
+                            });
+                        }
+                    })
+                    .fail(err => {
+                        const errMsg = JSON.parse(err);
+                        this.loadSecurityConfig();
+                        this.setState({
+                            saving: false
+                        });
+                        let msg = errMsg.desc;
+                        if ('info' in errMsg) {
+                            msg = errMsg.desc + " - " + errMsg.info;
+                        }
+                        this.props.addNotification(
+                            "error",
+                            `Error updating security RSA configuration - ${msg}`
+                        );
+                    });
+        }
+
         if (cmd.length > 5) {
             log_cmd("saveSecurityConfig", "Applying security config change", cmd);
             const msg = "Successfully updated security configuration.";
@@ -817,25 +1003,6 @@ export class Security extends React.Component {
                 configPage =
                     <div className="ds-margin-bottom-md">
                         <Form isHorizontal autoComplete="off">
-                            <Grid
-                                title="This parameter can be used to restrict the Directory Server instance to a single IP interface (hostname, or IP address).  This parameter specifically sets what interface to use for TLS traffic.  Requires restart. (nsslapd-securelistenhost)."
-                            >
-                                <GridItem className="ds-label" span={3}>
-                                    Secure Listen Host
-                                </GridItem>
-                                <GridItem span={8}>
-                                    <TextInput
-                                        value={this.state.secureListenhost}
-                                        type="text"
-                                        id="secureListenhost"
-                                        aria-describedby="horizontal-form-name-helper"
-                                        name="server-hostname"
-                                        onChange={(str, e) => {
-                                            this.handleChange(e);
-                                        }}
-                                    />
-                                </GridItem>
-                            </Grid>
                             <Grid
                                 title="The name, or nickname, of the server certificate inthe NSS database the server should use (nsSSLPersonalitySSL)."
                             >
@@ -956,6 +1123,25 @@ export class Security extends React.Component {
                                     </Select>
                                 </GridItem>
                             </Grid>
+                            <Grid
+                                title="This parameter can be used to restrict the Directory Server instance to a single IP interface (hostname, or IP address).  This parameter specifically sets what interface to use for TLS traffic.  Requires restart. (nsslapd-securelistenhost)."
+                            >
+                                <GridItem className="ds-label" span={3}>
+                                    Secure Listen Host
+                                </GridItem>
+                                <GridItem span={8}>
+                                    <TextInput
+                                        value={this.state.secureListenhost}
+                                        type="text"
+                                        id="secureListenhost"
+                                        aria-describedby="horizontal-form-name-helper"
+                                        name="server-hostname"
+                                        onChange={(str, e) => {
+                                            this.handleChange(e);
+                                        }}
+                                    />
+                                </GridItem>
+                            </Grid>
                             <Grid
                                 title="Require all connections use TLS (nsslapd-require-secure-binds)."
                             >
@@ -1071,7 +1257,10 @@ export class Security extends React.Component {
                                     serverId={this.props.serverId}
                                     CACerts={this.state.CACerts}
                                     ServerCerts={this.state.serverCerts}
+                                    ServerCSRs={this.state.serverCSRs}
+                                    ServerKeys={this.state.serverOrphanKeys}
                                     addNotification={this.props.addNotification}
+                                    certDir={this.props.certDir}
                                 />
                             </Tab>
                             <Tab eventKey={2} title={<TabTitleText>Cipher Preferences</TabTitleText>}>
