@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2022 Red Hat, Inc.
+# Copyright (C) 2023 Red Hat, Inc.
 # Copyright (C) 2019 William Brown <william@blackhats.net.au>
 # All rights reserved.
 #
@@ -107,7 +107,7 @@ def _get_backend(inst, be_name):
         if (is_a_dn(be_name) and str2dn(be_suffix) == str2dn(be_name)) or (not is_a_dn(be_name) and cn == be_name):
             return be
 
-    raise ValueError('Could not find backend suffix: {}'.format(name))
+    raise ValueError('Could not find backend suffix: {}'.format(be_name))
 
 
 def _get_index(inst, be_name, attr):
@@ -236,14 +236,17 @@ def backend_delete(inst, basedn, log, args, warn=True):
     dn = _search_backend_dn(inst, args.be_name)
     if dn is None:
         raise ValueError("Unable to find a backend with the name: ({})".format(args.be_name))
-    if warn and args.json is False:
-        _warn(dn, msg="Deleting %s %s" % (SINGULAR.__name__, dn))
+    if not args.ack:
+        log.info("""Not removing backend: if you are really sure add: --do-it""")
+    else:
+        if warn and args.json is False:
+            _warn(dn, msg="Deleting %s %s" % (SINGULAR.__name__, dn))
 
-    be = _get_backend(inst, args.be_name)
-    _recursively_del_backends(be)
-    be.delete()
+        be = _get_backend(inst, args.be_name)
+        _recursively_del_backends(be)
+        be.delete()
 
-    log.info("The database, and any sub-suffixes, were sucessfully deleted")
+        log.info("The database, and any sub-suffixes, were successfully deleted")
 
 
 def backend_import(inst, basedn, log, args):
@@ -256,7 +259,7 @@ def backend_import(inst, basedn, log, args):
     task = mc.import_ldif(ldifs=args.ldifs, chunk_size=args.chunks_size, encrypted=args.encrypted,
                           gen_uniq_id=args.gen_uniq_id, only_core=args.only_core, include_suffixes=args.include_suffixes,
                           exclude_suffixes=args.exclude_suffixes)
-    task.wait(timeout=None)
+    task.wait(timeout=args.timeout)
     result = task.get_exit_code()
     warning = task.get_task_warn()
 
@@ -266,7 +269,10 @@ def backend_import(inst, basedn, log, args):
         else:
             log.info("The import task has finished successfully, with warning code {}, check the logs for more detail".format(warning))
     else:
-        raise ValueError("Import task failed\n-------------------------\n{}".format(ensure_str(task.get_task_log())))
+        if result is None:
+            raise ValueError(f"Import task has not completed\n-------------------------\n{ensure_str(task.get_task_log())}")
+        else:
+            raise ValueError(f"Import task failed\n-------------------------\n{ensure_str(task.get_task_log())}")
 
 
 def backend_export(inst, basedn, log, args):
@@ -288,13 +294,16 @@ def backend_export(inst, basedn, log, args):
                           encrypted=args.encrypted, min_base64=args.min_base64, no_dump_uniq_id=args.no_dump_uniq_id,
                           replication=args.replication, not_folded=args.not_folded, no_seq_num=args.no_seq_num,
                           include_suffixes=args.include_suffixes, exclude_suffixes=args.exclude_suffixes)
-    task.wait(timeout=None)
+    task.wait(timeout=args.timeout)
     result = task.get_exit_code()
 
     if task.is_complete() and result == 0:
         log.info("The export task has finished successfully")
     else:
-        raise ValueError("Export task failed\n-------------------------\n{}".format(ensure_str(task.get_task_log())))
+        if result is None:
+            raise ValueError(f"Export task did not complete\n-------------------------\n{ensure_str(task.get_task_log())}")
+        else:
+            raise ValueError(f"Export task failed\n-------------------------\n{ensure_str(task.get_task_log())}")
 
 
 def is_db_link(inst, rdn):
@@ -815,9 +824,14 @@ def backend_compact(inst, basedn, log, args):
     if args.only_changelog:
         task_properties = {'justChangelog': 'yes'}
     task.create(properties=task_properties)
-    task.wait()
-    if task.get_exit_code() != 0:
-        raise ValueError("Failed to create Database Compaction Task")
+    task.wait(timeout=args.timeout)
+    result = task.get_exit_code()
+    if result != 0:
+        if result is None:
+            raise ValueError("Database Compaction Task has not completed")
+        else:
+            raise ValueError(f"Database Compaction Task failed, error: {result}")
+
     log.info("Successfully started Database Compaction Task")
 
 
@@ -1103,6 +1117,8 @@ def create_parser(subparsers):
                                help="Specifies the suffixes or the subtrees to be included")
     import_parser.add_argument('-x', '--exclude-suffixes', nargs='+',
                                help="Specifies the suffixes to be excluded")
+    import_parser.add_argument('--timeout', type=int, default=0,
+                               help="Set a timeout to wait for the export task.  Default is 0 (no timeout)")
 
     #######################################################
     # Export LDIF
@@ -1132,6 +1148,8 @@ def create_parser(subparsers):
                                help="Specifies the suffixes or the subtrees to be included")
     export_parser.add_argument('-x', '--exclude-suffixes', nargs='+',
                                help="Specifies the suffixes to be excluded")
+    export_parser.add_argument('--timeout', default=0, type=int,
+                               help="Set a timeout to wait for the export task.  Default is 0 (no timeout)")
 
     #######################################################
     # Create a new backend database
@@ -1152,6 +1170,9 @@ def create_parser(subparsers):
     delete_parser = subcommands.add_parser('delete', help='Delete a backend database')
     delete_parser.set_defaults(func=backend_delete)
     delete_parser.add_argument('be_name', help='The backend name or suffix')
+    delete_parser.add_argument('--do-it', dest="ack",
+                               help="Remove backend and its subsuffixes",
+                               action='store_true', default=False)
 
     #######################################################
     # Get Suffix Tree (for use in web console)
@@ -1165,3 +1186,5 @@ def create_parser(subparsers):
     compact_parser = subcommands.add_parser('compact-db', help='Compact the database and the replication changelog')
     compact_parser.set_defaults(func=backend_compact)
     compact_parser.add_argument('--only-changelog', action='store_true', help='Compacts only the replication change log')
+    compact_parser.add_argument('--timeout', default=0, type=int,
+                                help="Set a timeout to wait for the compaction task.  Default is 0 (no timeout)")
